@@ -65,6 +65,8 @@ static void dump_parameters(FILE *fp)
       char sss[GLP_MAX_NAME+1];
       glp_get_rpar("c0", &rrr);
       fprintf(fp, "c0             = %.8g\n", rrr);
+      glp_get_cpar("fn_gener", sss);
+      fprintf(fp, "fn_gener       = %s\n", sss);
       glp_get_ipar("mip_branch", &iii);
       fprintf(fp, "mip_branch     = %s\n",
          iii == GLP_FIRST ? "FIRST" : iii == GLP_LAST ? "LAST" :
