@@ -23,7 +23,6 @@
 -- Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 ----------------------------------------------------------------------*/
 
-#include <assert.h>
 #include <stddef.h>
 #include "glpapi.h"
 
@@ -68,7 +67,7 @@ int glp_set_name(int what, char *name)
             return 1;
          }
          node = find_by_key(glp->row_tab, glp->this_row->name);
-         assert(node != NULL);
+         insist(node != NULL);
          delete_node(glp->row_tab, node);
          delete_str(glp->this_row->name);
          glp->this_row->name = row_name;
@@ -87,7 +86,7 @@ int glp_set_name(int what, char *name)
             return 1;
          }
          node = find_by_key(glp->col_tab, glp->this_col->name);
-         assert(node != NULL);
+         insist(node != NULL);
          delete_node(glp->col_tab, node);
          delete_str(glp->this_col->name);
          glp->this_col->name = col_name;
