@@ -23,8 +23,8 @@
 -- Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 ----------------------------------------------------------------------*/
 
-#include <assert.h>
 #include "glpimg.h"
+#include "glpset.h"
 
 /*----------------------------------------------------------------------
 -- set_imgval.c - set pixel to given value.
@@ -108,7 +108,7 @@ void set_imgval(IMG *img, int x, int y, int val)
             *byte = (char)(val & 255);
             break;
          default:
-            assert(img->type != img->type);
+            insist(img->type != img->type);
       }
 skip: return;
 }
