@@ -23,7 +23,6 @@
 -- Foundation, 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 ----------------------------------------------------------------------*/
 
-#include <assert.h>
 #include <math.h>
 #include <stddef.h>
 #include "glpgel.h"
@@ -135,7 +134,7 @@ int update_rfi(RFI *rfi, int p)
       if (rfi->reid && k1 < k2)
       {  min_bump(rfi->P, rfi->V, rfi->Q, &k1, &k2, rfi->rs, rfi->cs,
             (int *)rfi->col, (int *)rfi->work);
-         assert(k1 <= k2);
+         insist(k1 <= k2);
       }
       /* now k1 <= k2; if k1 < k2, the matrix Unew should be transformed
          to the upper diagonal form by means of special version of
@@ -161,7 +160,7 @@ int update_rfi(RFI *rfi, int p)
                   &dum, rfi->work);
                break;
             default:
-               assert(rfi->tech != rfi->tech);
+               insist(rfi->tech != rfi->tech);
          }
          /* if gaussian elimination routine returned detected that the
             matrix U is near to singular or inaccurate, RFI should be
