@@ -67,6 +67,10 @@ struct GLPWKSP
       /* --- control parameters segment --- */
       double c0;
       /* the constant term of the objective function */
+      STR *fn_gener; /* used by the glp_read_lpm routine */
+      /* this character string specifies name of the file to which the
+         glp_read_lpm routine should output generated LP/MIP problem in
+         plain text format; NULL means no output */
       int mip_branch; /* used by MIP solver */
       /* this parameter specifies what branching technique should be
          used by the solver:
@@ -156,7 +160,7 @@ struct GLPWKSP
       /* solution status:
          GLP_UNDEF  - solution status is undefined
          GLP_OPT    - solution is optimal
-         GLP_FEAS   - soultion is feasible
+         GLP_FEAS   - solution is feasible
          GLP_INFEAS - solution is infeasible
          GLP_NOFEAS - problem has no feasible solution
          GLP_UNBND  - problem has unbounded solution
