@@ -30,22 +30,33 @@
 #include "glpset.h"
 
 /*----------------------------------------------------------------------
--- This program is a stand-alone LP/MIP solver based on the revised
--- simplex method and the branch-and-bound procedure.
+-- This program is a stand-alone LP/MIP solver. For pure LP problems
+-- either the revised simplex method or the primal dual interior-point
+-- method can be used. For MIP problems the branch-and-bound procedure
+-- based on the revised simplex method is used.
 ----------------------------------------------------------------------*/
 
-static char *version = "GLPSOL -- GLPK LP/MIP Solver, Version 2.3";
+static char *version = "GLPSOL -- GLPK LP/MIP Solver, Version 2.4";
 
-static char *dat_fname = "";
-/* name of the input text file which contains LP problem data in MPS or
-   LPM format */
+static char *in_fname = "";
+/* name of the input text file */
 
-static int dat_fmt = 0;
-/* format of the input text file (0 means MPS, 1 means LPM) */
+static int format = 'M';
+/* format of the input text file:
+   'M' - MPS
+   'L' - GLPK/L */
+
+static int check = 0;
+/* if this flag is set, only input data checking is required */
+
+static int method = 'S';
+/* the method which should be used for solving a problem:
+   'S' - simplex method
+   'I' - interior point method */
 
 static char *out_fname = "";
-/* name of the output text file to which the final basis solution will
-   be sent using printable format (optional) */
+/* name of the output text file to which the final solution should be
+   sent using printable format (optional) */
 
 static int nomip = 0;
 /* if this flag is set, the solver considers all integer variables as
@@ -58,35 +69,54 @@ static int nomip = 0;
 -- required by the GNU Coding Standards. */
 
 static void display_help(char *my_name)
-{     print("Usage: %s [options...] data-file", my_name);
+{     print("Usage: %s [options...] filename", my_name);
       print("");
       print("General options:");
-      print("   --mps             read LP data using MPS format (defaul"
-         "t)");
-      print("   --lpm             read LP data using LPM format");
-      print("   --min             objective function should be minimize"
-         "d (default)");
-      print("   --max             objective function should be maximize"
-         "d");
-      print("   --ini             only initial solution is required");
-      print("   --any             any feasible solution is required");
-      print("   --fin             final solution is required (default)")
+      print("   --mps             read LP/MIP data using MPS format (de"
+         "fault)");
+      print("   --lpm             read LP/MIP model written in GLPK/L m"
+         "odeling");
+      print("                     language");
+      print("   --check           do not solve the problem, check input"
+         " data only");
+      print("   --gener filename  send generated LP/MIP problem to file"
+         "name using");
+      print("                     plain text format (has sense for --lp"
+         "m only)");
+      print("   --min             minimize the objective function");
+      print("   --max             maximize the objective function");
+      print("   --ini             find initial solution only");
+      print("   --any             find any feasible solution");
+      print("   --fin             find final solution (default)");
+      print("   --simplex         use revised simplex method (default)")
          ;
+      print("   --interior        use interior point method (for pure L"
+         "P only)");
       print("   -o filename, --output filename");
       print("                     send solution to filename using print"
          "able format");
+      print("   --scale           scale the problem (default)");
+      print("   --noscale         do not scale the problem");
+      print("   --sum             enable multiplets in constraint matri"
+         "x (will be");
+      print("                     replaced by their sum)");
+      print("   --nosum           disable multiplets in constraint matr"
+         "ix (default)");
+      print("   -h, --help        display this help information and exi"
+         "t");
+      print("   -v, --version     display program version and exit");
+      print("");
+      print("Options specific to simplex method:");
       print("   --textbook        use options --noscale, --efi, --prima"
          "l, --nosteep,");
       print("                     --norelax, --first, and --lifo by def"
          "ault");
-      print("   --scale           scale the problem (default)");
-      print("   --noscale         do not scale the problem");
       print("   --efi             use EFI");
-      print("   --rfi-bg          use RFI + Bartels & Golub updating te"
-         "chnique");
+      print("   --rfi-bg          use RFI + Bartels and Golub updating "
+         "technique");
       print("                     (default)");
-      print("   --rfi-ft          use RFI + Forrest & Tomlin updating t"
-         "echnique");
+      print("   --rfi-ft          use RFI + Forrest and Tomlin updating"
+         " technique");
       print("   --primal          find feasible solution using primal s"
          "implex");
       print("                     (default)");
@@ -99,31 +129,26 @@ static void display_help(char *my_name)
          "ult)");
       print("   --norelax         use standard \"textbook\" ratio test")
          ;
-      print("   --sum             enable multiplets in constraint matri"
-         "x (will be");
-      print("                     replaced by their sum)");
-      print("   --nosum           disable multiplets in constraint matr"
-         "ix (default)");
-      print("   -h, --help        display this help information and exi"
-         "t");
-      print("   -v, --version     display program version and exit");
       print("");
       print("Options specific to MIP:");
       print("   --nomip           consider all integer variables as con"
-         "tinuous (this");
-      print("                     allows solving MIP problem as pure LP"
-         ")");
+         "tinuous");
+      print("                     (this allows solving MIP problem as p"
+         "ure LP)");
       print("   --first           branch on first integer variable");
       print("   --last            branch on last integer variable");
       print("   --drtom           branch using heuristic by Driebeck an"
          "d Tomlin");
       print("                     (default)");
-      print("   --fifo            backtrack using FIFO strategy");
-      print("   --lifo            backtrack using LIFO strategy");
+      print("   --fifo            backtrack using FIFO heuristic");
+      print("   --lifo            backtrack using LIFO heuristic");
       print("   --bestp           backtrack using the best projection h"
          "euristic");
       print("                     (default)");
       print("");
+      print("See the official GNU webpage dedicated to GLPK at");
+      print("<http://www.gnu.org/software/glpk/glpk.html>");
+      print("");
       print("Please, report bugs to <bug-glpk@gnu.org>");
       exit(EXIT_SUCCESS);
       /* no return */
@@ -151,37 +176,60 @@ static void display_version(void)
 --
 -- This routine processes parameters specified in command line. */
 
+#define param(str) (strcmp(argv[k], str) == 0)
+
 static void process_cmdline(int argc, char *argv[])
 {     int k;
       for (k = 1; k < argc; k++)
-      {  if (strcmp(argv[k], "--mps") == 0)
-            dat_fmt = 0;
-         else if (strcmp(argv[k], "--lpm") == 0)
-            dat_fmt = 1;
-         else if (strcmp(argv[k], "--min") == 0)
-            glp_set_ipar("obj_dir", GLP_MIN);
-         else if (strcmp(argv[k], "--max") == 0)
-            glp_set_ipar("obj_dir", GLP_MAX);
-         else if (strcmp(argv[k], "--ini") == 0)
-            glp_set_ipar("option", GLP_INI);
-         else if (strcmp(argv[k], "--any") == 0)
-            glp_set_ipar("option", GLP_ANY);
-         else if (strcmp(argv[k], "--fin") == 0)
-            glp_set_ipar("option", GLP_FIN);
-         else if (strcmp(argv[k], "-o") == 0 ||
-                  strcmp(argv[k], "--output") == 0)
+      {  if (param("--mps"))
+            format = 'M';
+         else if (param("--lpm"))
+            format = 'L';
+         else if (param("--check"))
+            check = 1;
+         else if (param("--gener"))
          {  k++;
-            if (k == argc)
-            {  error("No output file name specified");
+            if (k == argc || argv[k][0] == '\0' || argv[k][0] == '-')
+            {  error("No gener file name specified");
                exit(EXIT_FAILURE);
             }
-            if (out_fname[0] != '\0')
-            {  error("Only one output file allowed");
+            glp_set_cpar("fn_gener", argv[k]);
+         }
+         else if (param("--min"))
+            glp_set_ipar("obj_dir", GLP_MIN);
+         else if (param("--max"))
+            glp_set_ipar("obj_dir", GLP_MAX);
+         else if (param("--ini"))
+            glp_set_ipar("option", GLP_INI);
+         else if (param("--any"))
+            glp_set_ipar("option", GLP_ANY);
+         else if (param("--fin"))
+            glp_set_ipar("option", GLP_FIN);
+         else if (param("--simplex"))
+            method = 'S';
+         else if (param("--interior"))
+            method = 'I';
+         else if (param("-o") || param("--output"))
+         {  k++;
+            if (k == argc || argv[k][0] == '\0' || argv[k][0] == '-')
+            {  error("No output file name specified");
                exit(EXIT_FAILURE);
             }
             out_fname = argv[k];
          }
-         else if (strcmp(argv[k], "--textbook") == 0)
+         else if (param("--scale"))
+            glp_set_ipar("scale", GLP_YES);
+         else if (param("--noscale"))
+            glp_set_ipar("scale", GLP_NO);
+         else if (param("--sum"))
+            glp_set_ipar("sum_aij", GLP_YES);
+         else if (param("--nosum"))
+            glp_set_ipar("sum_aij", GLP_NO);
+         else if (param("-h") || param("--help"))
+            display_help(argv[0]);
+         else if (param("-v") || param("--version"))
+            display_version();
+         else if (param("--textbook"))
          {  glp_set_ipar("scale", GLP_NO);
             glp_set_ipar("spx_form", GLP_EFI);
             glp_set_ipar("spx_use_dual", GLP_NO);
@@ -190,51 +238,37 @@ static void process_cmdline(int argc, char *argv[])
             glp_set_ipar("branch", GLP_FIRST);
             glp_set_ipar("btrack", GLP_LIFO);
          }
-         else if (strcmp(argv[k], "--scale") == 0)
-            glp_set_ipar("scale", GLP_YES);
-         else if (strcmp(argv[k], "--noscale") == 0)
-            glp_set_ipar("scale", GLP_NO);
-         else if (strcmp(argv[k], "--efi") == 0)
+         else if (param("--efi"))
             glp_set_ipar("spx_form", GLP_EFI);
-         else if (strcmp(argv[k], "--rfi-bg") == 0)
+         else if (param("--rfi-bg"))
             glp_set_ipar("spx_form", GLP_RFI_BG);
-         else if (strcmp(argv[k], "--rfi-ft") == 0)
+         else if (param("--rfi-ft"))
             glp_set_ipar("spx_form", GLP_RFI_FT);
-         else if (strcmp(argv[k], "--primal") == 0)
+         else if (param("--primal"))
             glp_set_ipar("spx_use_dual", GLP_NO);
-         else if (strcmp(argv[k], "--dual") == 0)
+         else if (param("--dual"))
             glp_set_ipar("spx_use_dual", GLP_YES);
-         else if (strcmp(argv[k], "--steep") == 0)
+         else if (param("--steep"))
             glp_set_ipar("spx_steep", GLP_YES);
-         else if (strcmp(argv[k], "--nosteep") == 0)
+         else if (param("--nosteep"))
             glp_set_ipar("spx_steep", GLP_NO);
-         else if (strcmp(argv[k], "--relax") == 0)
+         else if (param("--relax"))
             glp_set_ipar("spx_relax", GLP_YES);
-         else if (strcmp(argv[k], "--norelax") == 0)
+         else if (param("--norelax"))
             glp_set_ipar("spx_relax", GLP_NO);
-         else if (strcmp(argv[k], "--sum") == 0)
-            glp_set_ipar("sum_aij", GLP_YES);
-         else if (strcmp(argv[k], "--nosum") == 0)
-            glp_set_ipar("sum_aij", GLP_NO);
-         else if (strcmp(argv[k], "-h") == 0 ||
-                  strcmp(argv[k], "--help") == 0)
-            display_help(argv[0]);
-         else if (strcmp(argv[k], "-v") == 0 ||
-                  strcmp(argv[k], "--version") == 0)
-            display_version();
-         else if (strcmp(argv[k], "--nomip") == 0)
+         else if (param("--nomip"))
             nomip = 1;
-         else if (strcmp(argv[k], "--first") == 0)
+         else if (param("--first"))
             glp_set_ipar("mip_branch", GLP_FIRST);
-         else if (strcmp(argv[k], "--last") == 0)
+         else if (param("--last"))
             glp_set_ipar("mip_branch", GLP_LAST);
-         else if (strcmp(argv[k], "--drtom") == 0)
+         else if (param("--drtom"))
             glp_set_ipar("mip_branch", GLP_DRTOM);
-         else if (strcmp(argv[k], "--fifo") == 0)
+         else if (param("--fifo"))
             glp_set_ipar("mip_btrack", GLP_FIFO);
-         else if (strcmp(argv[k], "--lifo") == 0)
+         else if (param("--lifo"))
             glp_set_ipar("mip_btrack", GLP_LIFO);
-         else if (strcmp(argv[k], "--bestp") == 0)
+         else if (param("--bestp"))
             glp_set_ipar("mip_btrack", GLP_BESTP);
          else if (argv[k][0] == '-' ||
                  (argv[k][0] == '-' && argv[k][1] == '-'))
@@ -243,11 +277,11 @@ static void process_cmdline(int argc, char *argv[])
             exit(EXIT_FAILURE);
          }
          else
-         {  if (dat_fname[0] != '\0')
+         {  if (in_fname[0] != '\0')
             {  error("Only one input file allowed");
                exit(EXIT_FAILURE);
             }
-            dat_fname = argv[k];
+            in_fname = argv[k];
          }
       }
       return;
@@ -267,33 +301,56 @@ int main(int argc, char *argv[])
       /* process command line parameters */
       process_cmdline(argc, argv);
       /* read LP problem data from the input file */
-      if (dat_fname[0] == '\0')
-      {  error("No input file name specified");
+      if (in_fname[0] == '\0')
+      {  error("No input file name specified; try %s --help", argv[0]);
          exit(EXIT_FAILURE);
       }
       glp_get_ipar("option", &option);
-      if (dat_fmt == 0)
-      {  if (glp_read_mps(dat_fname) != 0)
-         {  error("Can't read MPS file");
-            exit(EXIT_FAILURE);
-         }
-      }
-      else
-      {  if (glp_read_lpm(dat_fname) != 0)
-         {  error("Can't read LPM file");
-            exit(EXIT_FAILURE);
-         }
+      switch (format)
+      {  case 'M':
+            if (glp_read_mps(in_fname) != 0)
+            {  error("MPS file processing error");
+               exit(EXIT_FAILURE);
+            }
+            break;
+         case 'L':
+            if (glp_read_lpm(in_fname) != 0)
+            {  error("Model description processing error");
+               exit(EXIT_FAILURE);
+            }
+            break;
+         default:
+            insist(format != format);
       }
       glp_set_ipar("option", option);
-      /* solve LP problem */
+      if (check) goto skip;
+      /* solve LP/MIP problem */
       t_start = clock();
       glp_get_ipar("nc_int", &nc_int);
-      ret = ((nomip || nc_int == 0) ? glp_simplex() : glp_integer());
+      switch (method)
+      {  case 'S':
+            if (nc_int == 0 || nomip)
+               ret = glp_simplex();
+            else
+               ret = glp_integer();
+            break;
+         case 'I':
+            if (nc_int == 0 || nomip)
+               ret = glp_interior();
+            else
+            {  error("Unable to solve MIP problem using interior point "
+                  "method");
+               exit(EXIT_FAILURE);
+            }
+            break;
+         default:
+            insist(method != method);
+      }
       switch (ret)
       {  case 0:
             break;
          case 1:
-            error("Can't solve the problem because of errors");
+            error("Unable to solve the problem because of errors");
             exit(EXIT_FAILURE);
          case 2:
             error("Optimization terminated because of solver failure");
@@ -308,10 +365,10 @@ int main(int argc, char *argv[])
          (double)mem_tpeak / (1024.0 * 1024.0), mem_tpeak);
       /* write final solution found by the solver */
       if (out_fname[0] != '\0' && glp_print_sol(out_fname) != 0)
-      {  error("Can't write solution");
+      {  error("Unable to write problem solution");
          exit(EXIT_FAILURE);
       }
-      /* terminate GLPK API */
+skip: /* terminate GLPK API */
       glp_terminate();
       /* exit to the control program */
       return 0;
